/* EasyCODE V8 */
/* EasyCODE ( 0 
Boot_loader.c */
/* EasyCODE_STRUCTURED_COMMENT_START */
/*
#pragma opt-
EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
*/
/* EasyCODE_STRUCTURED_COMMENT_END */
#include <system.h>
#include <sys_cpu.h>
#include <string.h>
typedef u8 uint8_t;
typedef u16 uint16_t;
typedef u32 uint32_t;
#include <sp_driver.h>
#include <flash_memory.h>
/* EasyCODE - */
#define cBufferLength 256
/* EasyCODE - */
#define cFlashPageLength 512 /*Bytes. Page length of CPU*/
/* EasyCODE - */
//Command types
#define cFlashData 0x00
#define cEndOfHexFile 0x01
#define cAddressOffset 0x02
/* EasyCODE - */
static volatile u8 flashBuffer[cBufferLength];
/* EasyCODE - */
static volatile u16 page=0; //Holds the next page to read from dataflash
/* EasyCODE - */
static volatile u32 bytesInBuffer=0, bufferIndex=0;
/* EasyCODE - */
void (*jump_to_app)(void) = 0x0000;
/* EasyCODE [ 
sendTextMessage (DEBUG) */
#ifdef _USE_DEBUG
#endif 
/* EasyCODE ] */
/* EasyCODE F */
static void sendText(u8* text)
{
    while ( *text != '\0' )
    {
        USARTC0.STATUS &= USART_DREIF_bm; //DRE
        /* EasyCODE - */
        USARTC0.DATA=*text;
        text++;
        while ( !(USARTC0.STATUS & USART_DREIF_bm) )
        {
            ;
        }
    }
}

/* EasyCODE F */
static void sendTextMessage(u8* text)
{
    sendText("BootLoader: ");
    /* EasyCODE - */
    sendText(text);
}

/* EasyCODE ( 0 
HEX_DEBUG */
#ifdef _USE_HEXOUTPUT
    /* EasyCODE F */
    static void sendHexMessage(u8* text, u16 length)
    {
        u16 i=0;
        while ( i < length )
        {
            USARTC0.STATUS &= USART_DREIF_bm; //DRE
            /* EasyCODE - */
            USARTC0.DATA=*text;
            i++;
            text++;
            while ( !(USARTC0.STATUS & USART_DREIF_bm) )
            {
                ;
            }
        }
    }

#endif 
/* EasyCODE ) */
/* EasyCODE [ 
Private functions */
/* EasyCODE ( 0 
ResetFlashHandling */
/* EasyCODE F */
static void ResetFlashHandling(u16 startPage)
{
    bytesInBuffer=0;
    bufferIndex=0;
    page=startPage;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
GetDataFromFlashMemory

Reads data from flash memory, using a buffer in RAM
This function assumes that the length <= cBufferLength */
/* EasyCODE F */
static void GetDataFromFlashMemory(u8 numBytes, u8* outData)
{
    u16 realPage;
    realPage=(page%cFirmwareAreaLength)+cFirmwareAreaStart;
    if ( bytesInBuffer==0 )
    {
        Read_nv_par_array(realPage, 0, cBufferLength, flashBuffer);
        page++;
        /* EasyCODE - */
        /* EasyCODE_STRUCTURED_COMMENT_START */
        /*
        sendTextMessage("page incremented!\r\n");
        EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
        */
        /* EasyCODE_STRUCTURED_COMMENT_END */
        bytesInBuffer=cBufferLength;
        bufferIndex=0;
    }
    if ( numBytes <= bytesInBuffer )
    {
        memcpy((void*)outData, (void*)&flashBuffer[bufferIndex], numBytes);
        bufferIndex+=numBytes;
        bytesInBuffer-=numBytes;
    }
    else
    {
        memcpy((void*)outData, (void*)&flashBuffer[bufferIndex], bytesInBuffer);
        numBytes-=bytesInBuffer;
        /* EasyCODE - */
        realPage=(page%cFirmwareAreaLength)+cFirmwareAreaStart;
        Read_nv_par_array(realPage, 0, cBufferLength, flashBuffer);
        page++;
        /* EasyCODE - */
        /* EasyCODE_STRUCTURED_COMMENT_START */
        /*
        sendTextMessage("page incremented!\r\n");
        EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
        */
        /* EasyCODE_STRUCTURED_COMMENT_END */
        memcpy((void*)&outData[bytesInBuffer], (void*)flashBuffer, numBytes);
        bytesInBuffer=cBufferLength-numBytes;
        bufferIndex=numBytes;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
LoadDataIntoBuffer

Loads one command from the data flash to the outputparameters
Returns TRUE if CRC is correct, otherwise FALSE */
/* EasyCODE F */
static u8 LoadDataIntoBuffer(u8* cmdType, u8* cmdData, u8* cmdByteCount, u32* cmdAddress)
{
    volatile u8 header[4], i, cmdChecksum=0, tempU8;
    /* EasyCODE - */
    u8 temp;
    /* EasyCODE ( 0 
    HEX_DEBUG */
    #ifdef _USE_HEXOUTPUT
        u8 test2[]={0xDE, 0xAD, 0xBE, 0xEF, 0x00};
        //sendHexMessage(test2, 4);
    #endif 
    /* EasyCODE ) */
    GetDataFromFlashMemory(4, header);
    /* EasyCODE ( 0 
    HEX_DEBUG */
    #ifdef _USE_HEXOUTPUT
        sendHexMessage(header, 4);
    #endif 
    /* EasyCODE ) */
    *cmdByteCount=header[0];
    GetDataFromFlashMemory(*cmdByteCount, cmdData);
    /* EasyCODE ( 0 
    HEX_DEBUG */
    #ifdef _USE_HEXOUTPUT
        sendHexMessage(cmdData, (u16)cmdByteCount);
    #endif 
    /* EasyCODE ) */
    *cmdType=header[3];
    /* EasyCODE - */
    (*cmdAddress)=header[1];
    (*cmdAddress)= (*cmdAddress) << 8;
    (*cmdAddress)= (*cmdAddress) | header[2];
    for ( i=0;i<4;i++ )
    {
        cmdChecksum+=header[i];
    }
    for ( i=0;i<(*cmdByteCount);i++ )
    {
        cmdChecksum+=cmdData[i];
    }
    GetDataFromFlashMemory(1, &tempU8);
    cmdChecksum+=tempU8;
    /* EasyCODE ( 0 
    HEX_DEBUG */
    #ifdef _USE_HEXOUTPUT
        sendHexMessage(&cmdChecksum, 1);
        /* EasyCODE - */
        sendHexMessage(test2, 4);
        /* EasyCODE < */
        while(1)
        {
        	temp++;
        }
        /* EasyCODE > */
        if ( cmdChecksum!=0 )
        {
            sendHexMessage(test2, 4);
            sendHexMessage(header, 4);
            sendHexMessage(cmdData, (u16)cmdByteCount);
            sendHexMessage(&cmdChecksum, 1);
            sendHexMessage(test2, 4);
        }
    #endif 
    /* EasyCODE ) */
    return cmdChecksum==0;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
ClearFlashBuffer

Clears buffer to CPU flash */
/* EasyCODE F */
static void ClearFlashBuffer(void)
{
    SP_EraseFlashBuffer();
    SP_WaitForSPM();
}

/* EasyCODE ) */
/* EasyCODE [ 
StoreFlashPage

Stores buffer in CPU flash */
/* EasyCODE F */
static void StoreFlashPage(u32 address)
{
    SP_WriteApplicationPage(address);
    /* EasyCODE - */
    SP_WaitForSPM();
    /* EasyCODE - */
    ClearFlashBuffer();
}

/* EasyCODE ] */
/* EasyCODE [ 
PerformUpdate */
/* EasyCODE F */
static void PerformUpdate(void)
{
    u8 cmdDataBuffer[32], cmdByteCount=0, count;
    u32 cmdAddress=0, tempAddress;
    u16 flashBufferIndex;
    u32 addressOffset=0, tempOffset;
    /* EasyCODE - */
    u8 buffer[10];
    u8 i;
    /* EasyCODE - */
    u8  cmdType=!cEndOfHexFile, *cmdDataPtr=cmdDataBuffer;
    u16 data;
    /* EasyCODE - */
    cmdByteCount=0;
    /* EasyCODE - */
    ClearFlashBuffer();
    while ( cmdType != cEndOfHexFile )
    {
        flashBufferIndex=0;
        while ( (flashBufferIndex < cFlashPageLength) &&
                (cmdType!=cEndOfHexFile) )
        {
            if ( cmdByteCount )
            {
                /* EasyCODE [ 
                Swap data bytes */
                data=*cmdDataPtr;
                cmdDataPtr++;
                data |= ( (u16)(*cmdDataPtr) << 8);
                cmdDataPtr++;
                /* EasyCODE ] */
                /* EasyCODE [ 
                store to Flashbuffer */
                SP_LoadFlashWord(flashBufferIndex, data);
                SP_WaitForSPM();
                /* EasyCODE ] */
                /* EasyCODE [ 
                HEX DEbug */
                /* EasyCODE_STRUCTURED_COMMENT_START */
                /*
                buffer[0]=(u8)((flashBufferIndex>>24)&0xFF);
                buffer[1]=(u8)((flashBufferIndex>>16)&0xFF);
                buffer[2]=(u8)((flashBufferIndex>>8)&0xFF);
                buffer[3]=(u8)(flashBufferIndex&0xFF);
                buffer[4]=(u8)((data>>8)&0xFF);
                buffer[5]=(u8)(data&0xFF);
                buffer[6]=0xFF;
                buffer[7]=0xFF;
                EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
                buffer[8]=0xFF;
                buffer[9]=0xFF;
                for ( i=0;i<8;i++ )
                {
                    USARTC0.STATUS &= USART_DREIF_bm; //DRE
                    EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
                    USARTC0.DATA=buffer[i];
                    while ( !(USARTC0.STATUS & USART_DREIF_bm) )
                    {
                        ;
                    }
                }
                */
                /* EasyCODE_STRUCTURED_COMMENT_END */
                /* EasyCODE ] */
                /* EasyCODE [ 
                update counter and pointer */
                cmdByteCount-=2;
                flashBufferIndex+=2;
                /* EasyCODE - */
                cmdAddress+=2;
                /* EasyCODE ] */
            }
            else
            {
                LoadDataIntoBuffer(&cmdType, cmdDataBuffer, &cmdByteCount, &tempAddress);
                /* EasyCODE - */
                cmdDataPtr=cmdDataBuffer;
                switch ( cmdType )
                {
                case cFlashData :
                    /* EasyCODE ( 0 
                    New data */
                    if ( tempAddress != cmdAddress )
                    {
                        if ( (tempAddress & 0xFE00) != (cmdAddress & 0xFE00) )
                        {
                            //New page
                            if ( flashBufferIndex )
                            {
                                if ( flashBufferIndex < cFlashPageLength )
                                {
                                    StoreFlashPage(addressOffset+cmdAddress);
                                }
                                else
                                {
                                    StoreFlashPage(addressOffset+cmdAddress-cFlashPageLength);
                                }
                                flashBufferIndex=0;
                            }
                        }
                        else
                        {
                            //Jump inside a page
                        }
                        cmdAddress=tempAddress;
                    }
                    else
                    {
                        //consecutive block
                    }
                    flashBufferIndex= cmdAddress & 0x01FF;
                    /* EasyCODE ) */
                    break;
                case cAddressOffset :
                    /* EasyCODE ( 0 
                    New address offset */
                    tempOffset=0;
                    for ( count=0; count < cmdByteCount; count++ )
                    {
                        tempOffset <<=8;
                        tempOffset |= cmdDataBuffer[count];
                    }
                    tempOffset <<=4;
                    if ( tempOffset != addressOffset )
                    {
                        if ( flashBufferIndex )
                        {
                            if ( flashBufferIndex < cFlashPageLength )
                            {
                                StoreFlashPage(addressOffset+cmdAddress);
                            }
                            else
                            {
                                StoreFlashPage(addressOffset+cmdAddress-cFlashPageLength);
                            }
                            flashBufferIndex=0;
                        }
                        addressOffset=tempOffset;
                    }
                    else
                    {
                        //No change
                    }
                    cmdByteCount=0;
                    /* EasyCODE ) */
                    break;
                default:
                    /* EasyCODE ( 0 
                    End of data */
                    cmdByteCount=0;
                    if ( flashBufferIndex )
                    {
                        if ( flashBufferIndex < cFlashPageLength )
                        {
                            StoreFlashPage(addressOffset+cmdAddress);
                        }
                        else
                        {
                            StoreFlashPage(addressOffset+cmdAddress-cFlashPageLength);
                        }
                        flashBufferIndex=0;
                    }
                    /* EasyCODE ) */
                }
            }
        }
        if ( flashBufferIndex )
        {
            if ( flashBufferIndex < cFlashPageLength )
            {
                StoreFlashPage(addressOffset+cmdAddress);
            }
            else
            {
                StoreFlashPage(addressOffset+cmdAddress-cFlashPageLength);
            }
        }
    }
}

/* EasyCODE ] */
/* EasyCODE ] */
/* EasyCODE ( 0 
JumpToApplication */
/* EasyCODE F */
void JumpToApplication(void)
{
    SP_WaitForSPM();
    SP_LockSPM();
    /* EasyCODE - */
    // remap interrupts back to application section
    CCP = CCP_IOREG_gc;
    PMIC.CTRL = 0;
    RAMPZ=0;
    /* EasyCODE - */
    EIND=0;
    /* EasyCODE - */
    jump_to_app();
}

/* EasyCODE ) */
/* EasyCODE ( 0 
InitCPUClock */
/* EasyCODE F */
void InitCPUClock(void)
{
    //Init of CPU clock here
    /* EasyCODE - */
    //Set 32MHz Oscillator
    OSC.CTRL|=OSC_RC32MEN_bm;
    /* EasyCODE - */
    //Wait for stable CLK
    while ( !(OSC.STATUS & OSC_RC32MRDY_bm) )
    {
        ;
    }
    //needed before writing to protected IO register 
    CCP=CCP_IOREG_gc;
    //Set 32MHz CLK 
    CLK.CTRL=CLK_SCLKSEL_RC32M_gc;
    //needed before writing to protected IO register 
    CCP=CCP_IOREG_gc;
    #if _MCU_CLOCK_FREQUENCY_ == 16000000
        //Set prescaler to 2--> CLK=16MHz
        CLK.PSCTRL=CLK_PSADIV0_bm;
    #endif 
    //disable 2MHz oscillator 
    OSC.CTRL&=~OSC_RC2MEN_bm;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
CheckUpdateAvailable */
/* EasyCODE F */
u8 CheckUpdateAvailable(u16 startPage)
{
    volatile u8 cmdDataBuffer[32], cmdByteCount=0, count, result=TRUE;
    volatile u32 cmdAddress=0, tempAddress, flashBufferIndex;
    volatile u32 addressOffset=0, tempOffset;
    /* EasyCODE - */
    volatile u8  cmdType=!cEndOfHexFile, *cmdDataPtr=cmdDataBuffer;
    /* EasyCODE - */
    cmdByteCount=0;
    /* EasyCODE ( 0 
    DEBUG */
    #ifdef _USE_DEBUG
        sendTextMessage("reseting Flashhandler\r\n");
    #endif 
    /* EasyCODE ) */
    ResetFlashHandling(startPage);
    while ( (cmdType != cEndOfHexFile) &&
            result )
    {
        flashBufferIndex=0;
        while ( (flashBufferIndex < cFlashPageLength) &&
                (cmdType!=cEndOfHexFile) &&
                result )
        {
            if ( cmdByteCount )
            {
                cmdByteCount-=2;
                flashBufferIndex+=2;
                /* EasyCODE - */
                cmdAddress+=2;
                /* EasyCODE ( 0 
                DEBUG */
                #ifdef _USE_DEBUG
                    sendTextMessage("cmdByteCount > 0\r\n");
                #endif 
                /* EasyCODE ) */
            }
            else
            {
                /* EasyCODE ( 0 
                DEBUG */
                #ifdef _USE_DEBUG
                    sendTextMessage("cmdByteCount == 0\r\n");
                #endif 
                /* EasyCODE ) */
                if ( LoadDataIntoBuffer(&cmdType, cmdDataBuffer, &cmdByteCount, &tempAddress) )
                {
                    /* EasyCODE ( 0 
                    DEBUG */
                    #ifdef _USE_DEBUG
                        sendTextMessage("got into Buffer\r\n");
                    #endif 
                    /* EasyCODE ) */
                    cmdDataPtr=cmdDataBuffer;
                    switch ( cmdType )
                    {
                    case cFlashData :
                        /* EasyCODE ( 0 
                        New data */
                        if ( tempAddress != cmdAddress )
                        {
                            /* EasyCODE ( 0 
                            DEBUG */
                            #ifdef _USE_DEBUG
                                sendTextMessage("tempAddr != cmdAddr\r\n");
                            #endif 
                            /* EasyCODE ) */
                            if ( (tempAddress & 0xFE00) != (cmdAddress & 0xFE00) )
                            {
                                //New page
                                /* EasyCODE ( 0 
                                DEBUG */
                                #ifdef _USE_DEBUG
                                    sendTextMessage("tempAddr MSB != cmdAddr  MSB\r\n");
                                #endif 
                                /* EasyCODE ) */
                                if ( flashBufferIndex )
                                {
                                    flashBufferIndex=0;
                                }
                            }
                            else
                            {
                                //Jump inside a page
                                /* EasyCODE ( 0 
                                DEBUG */
                                #ifdef _USE_DEBUG
                                    sendTextMessage("tempAddr MSB = cmdAddr  MSB\r\n");
                                #endif 
                                /* EasyCODE ) */
                                flashBufferIndex= tempAddress & 0x01FF;
                            }
                            cmdAddress=tempAddress;
                        }
                        else
                        {
                            //consecutive block
                            /* EasyCODE ( 0 
                            DEBUG */
                            #ifdef _USE_DEBUG
                                sendTextMessage("tempAddr MSB = cmdAddr  MSB\r\n");
                            #endif 
                            /* EasyCODE ) */
                        }
                        /* EasyCODE ) */
                        /* EasyCODE ( 0 
                        DEBUG */
                        #ifdef _USE_DEBUG
                            sendTextMessage("exec cFlashData\r\n");
                        #endif 
                        /* EasyCODE ) */
                        break;
                    case cAddressOffset :
                        /* EasyCODE ( 0 
                        New address offset */
                        tempOffset=0;
                        for ( count=0; count < cmdByteCount; count++ )
                        {
                            tempOffset <<=8;
                            tempOffset |= cmdDataBuffer[count];
                        }
                        tempOffset <<=4;
                        if ( tempOffset != addressOffset )
                        {
                            if ( flashBufferIndex )
                            {
                                flashBufferIndex=0;
                            }
                            addressOffset=tempOffset;
                        }
                        else
                        {
                            //No change
                        }
                        cmdByteCount=0;
                        /* EasyCODE ) */
                        /* EasyCODE ( 0 
                        DEBUG */
                        #ifdef _USE_DEBUG
                            sendTextMessage("exec cAddressOffset\r\n");
                        #endif 
                        /* EasyCODE ) */
                        break;
                    case cEndOfHexFile :
                        //End of data
                        /* EasyCODE - */
                        cmdByteCount=0;
                        /* EasyCODE - */
                        flashBufferIndex=0;
                        /* EasyCODE ( 0 
                        DEBUG */
                        #ifdef _USE_DEBUG
                            sendTextMessage("exec EOF\r\n");
                        #endif 
                        /* EasyCODE ) */
                        break;
                    default:
                        //Unknown command
                        result=FALSE;
                        /* EasyCODE ( 0 
                        DEBUG */
                        #ifdef _USE_DEBUG
                            sendTextMessage("exec default\r\n");
                        #endif 
                        /* EasyCODE ) */
                    }
                }
                else
                {
                    result=FALSE;
                    /* EasyCODE ( 0 
                    DEBUG */
                    #ifdef _USE_DEBUG
                        sendTextMessage("Nothing in Buffer\r\n");
                    #endif 
                    /* EasyCODE ) */
                }
            }
        }
    }
    if ( !result )
    {
        SP_WaitForSPM();
        SP_LockSPM();
        SP_WaitForSPM();
    }
    return result;
}

/* EasyCODE ) */
/* EasyCODE [ 
UpdateFirmware */
/* EasyCODE F */
void UpdateFirmware(u16 startPage)
{
    u16 realPage, i;
    /* EasyCODE - */
    ResetFlashHandling(startPage);
    /* EasyCODE - */
    SP_EraseApplicationSection();    
    SP_WaitForSPM();
    /* EasyCODE ( 0 
    DEBUG */
    #ifdef _USE_DEBUG
        sendTextMessage("Performing update!\r\n");
    #endif 
    /* EasyCODE ) */
    PerformUpdate();
    /* EasyCODE - */
    SP_WaitForSPM();
    /* EasyCODE ( 0 
    DEBUG */
    #ifdef _USE_DEBUG
        sendTextMessage("Removing firmware!\r\n");
    #endif 
    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Remove firmware from data flash */
    /* EasyCODE_STRUCTURED_COMMENT_START */
    /*
    memset (flashBuffer, 0, cBufferLength);
    for ( i=startPage; i < page;i++ )
    {
        realPage=(i%cFirmwareAreaLength)+cFirmwareAreaStart;
        EasyCODE_COMMENT_START EasyCODE - EasyCODE_COMMENT_END
        Write_nv_par_array( realPage, 0, cBufferLength, flashBuffer);
    }
    */
    /* EasyCODE_STRUCTURED_COMMENT_END */
    /* EasyCODE ) */
    /* EasyCODE_STRUCTURED_COMMENT_START */
    /*
    CCP=CCP_IOREG_gc; //Enable write to reset register
    RST.CTRL=RST_SWRST_bm; //software reset
    */
    /* EasyCODE_STRUCTURED_COMMENT_END */
}

/* EasyCODE ] */
/* EasyCODE ) */
