/* EasyCODE V8 */
/* EasyCODE ( 0 */
/* EasyCODE_STRUCTURED_COMMENT_START */
/*
#ifdef _CHIP_ATXMEGA128A3_
    #include <xmega128a3.h>    // I/O Registers definitions
#endif 
#ifdef _CHIP_ATXMEGA128D3_
    #include <xmega128d3.h>    // I/O Registers definitions
#endif 
*/
/* EasyCODE_STRUCTURED_COMMENT_END */
#include <sys_cpu.h>
/* EasyCODE - */
#include <system.h>
#include <stdlib.h>
#include <spi_driver.h>
/* EasyCODE ( 0 
dataflash definitions */
#define write_flash 0
#define read_flash 1
#define fcmd_buf1_read 0xD4                // SPI compatible command
#define fcmd_buf2_read 0xD6                // SPI compatible command
#define fcmd_status_reg_read 0xD7            // SPI compatible command
#define fcmd_mainmem_page_read 0xD2            // SPI compatible command
#define fcmd_mainmem_to_buf1_xfer 0x53
#define fcmd_mainmem_to_buf2_xfer 0x55
#define fcmd_mainmem_to_buf1_cmp 0x60
#define fcmd_mainmem_to_buf2_cmp 0x61
#define fcmd_buf1_write 0x84
#define fcmd_buf2_write 0x87
#define fcmd_buf1_to_mainmem_prg_erase 0x83
#define fcmd_buf2_to_mainmem_prg_erase 0x86
#define fcmd_buf1_to_mainmem_prg_no_erase 0x88
#define fcmd_buf2_to_mainmem_prg_no_erase 0x89
#define fcmd_mainmem_prg_through_buf1 0x82
#define fcmd_mainmem_prg_through_buf2 0x85
#define fcmd_auto_page_rewr_through_buf1 0x58
#define fcmd_auto_page_rewr_through_buf2 0x59
/* EasyCODE ) */
/* EasyCODE C */
typedef struct SFlashMem
{
        u8 initSuccess;
}TFlashMem;
/* EasyCODE E */
static TFlashMem flashMem={0};
/* EasyCODE ( 0 
chipSelectLow */
/* EasyCODE F */
static inline void chipSelectLow(void)
{
    cChipSelect.OUTCLR=cPinMask;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
chipSelectHigh */
/* EasyCODE F */
static inline void chipSelectHigh(void)
{
    cChipSelect.OUTSET=cPinMask;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
WaitFlashReady */
/* EasyCODE F */
static void WaitFlashReady(void)
{
    u8 tempU8;
    /* EasyCODE - */
    chipSelectLow();
    /* EasyCODE - */
    tempU8=fcmd_status_reg_read;
    spi_transmitData( cFlashMemSPI, &tempU8, 1);
    tempU8=0;
    while ( !(tempU8 & 0x80) )
    {
        spi_receiveData( cFlashMemSPI, &tempU8, 1);
    }
    chipSelectHigh();
}

/* EasyCODE ) */
/* EasyCODE ( 0 
Flash_command */
/* EasyCODE F */
static void Flash_command(u8 command, u16 page, u16 address, u8 nr_dc_bytes, u16 nr_data_bytes, u8 data[], u8 mode)
{
    u8 cmdArray[4];
    WaitFlashReady();
    /* EasyCODE - */
    chipSelectLow();
    /* EasyCODE - */
    cmdArray[0]= command;
    cmdArray[1]= (page>>7)&0x00FF;
    cmdArray[2]= (page<<1 | address>>8)&0x00FF;
    cmdArray[3]=address&0x00FF;
    /* EasyCODE - */
    spi_transmitData( cFlashMemSPI,cmdArray, 4);
    if ( nr_dc_bytes )
    {
        spi_transmitData( cFlashMemSPI,cmdArray, nr_dc_bytes);
    }
    if ( nr_data_bytes &&
         (data != NULL) )
    {
        if ( mode==read_flash )
        {
            spi_receiveData( cFlashMemSPI,data, nr_data_bytes);
        }
        else
        {
            spi_transmitData( cFlashMemSPI,data, nr_data_bytes);
        }
    }
    chipSelectHigh();
}

/* EasyCODE ) */
/* EasyCODE ( 0 
Read_nv_par_array */
/* EasyCODE F */
void Read_nv_par_array(u16 nv_flash_page, u16 start_address, u16 data_length, u8 data_read[])      // read array from data_flash
{
    Flash_command(fcmd_mainmem_to_buf1_xfer, nv_flash_page, 0, 0, 0, data_read, write_flash);          // parameters from flash to buffer 1
    /* EasyCODE - */
    Flash_command(fcmd_buf1_read, 0, start_address, 1, data_length, data_read, read_flash);         // read data from buffer 1
}

/* EasyCODE ) */
/* EasyCODE ( 0 
Write_nv_par_array */
/* EasyCODE F */
void Write_nv_par_array(u16 nv_flash_page, u16 start_address, u16 data_length, u8 data_to_write[])       // write array to data_flash
{
    Flash_command(fcmd_mainmem_to_buf1_xfer, nv_flash_page, 0, 0, 0, data_to_write, read_flash);           // parameters from flash 
    //to buffer 1
    /* EasyCODE - */
    Flash_command(fcmd_buf1_write, 0, start_address, 0, data_length, data_to_write, write_flash);          // write data to buffer 1
    Flash_command(fcmd_buf1_to_mainmem_prg_erase, nv_flash_page, 0, 0, 0, data_to_write, write_flash);
}

/* EasyCODE ) */
/* EasyCODE ( 0 
InitFlashMemory */
/* EasyCODE F */
u8 InitFlashMemory(void)
{
    cChipSelect.DIRSET=cPinMask;
    /* EasyCODE - */
    flashMem.initSuccess = spi_initBlockingMaster(cFlashMemSPI, 20000, eSys_SPIMode0);
    /* EasyCODE - */
    flashMem.initSuccess |= spi_changeDataOrder(cFlashMemSPI, FALSE);
    /* EasyCODE - */
    chipSelectHigh();
    return flashMem.initSuccess;
}

/* EasyCODE ) */
/* EasyCODE ) */
