/* EasyCODE V8 */
/* EasyCODE ( 0 */
#include <system.h>
#include <stdlib.h>
#include <spi_driver.h>
#include <math.h>
/* EasyCODE [ 
SSPI_Buffer */
/* EasyCODE C */
typedef struct SSPI_Buffer
{
        u8* buffer;
        u16 length;
        u8* readPointer;
        u8* writePointer;
}TSPI_Buffer;
/* EasyCODE E */
/* EasyCODE ] */
/* EasyCODE [ 
SSPI_Manager */
/* EasyCODE C */
typedef struct SSPI_Manager
{
        TSPI_State state;
        TSPI_Buffer receiveBuffer;
        TSPI_Buffer transmitBuffer;
}TSPI_Manager;
/* EasyCODE E */
/* EasyCODE ] */
static TSPI_Manager spiManager[cSys_SPICount];
/* EasyCODE [ 
Private functions */
/* EasyCODE ( 0 
spi_blockingMasterEventHandler */
/* EasyCODE F */
static void spi_blockingMasterEventHandler(u8 aSPI, TSys_ReceivedData data)
{
    switch ( spiManager[aSPI].state.communicationState )
    {
    case eSPI_receiving :
        if ( spiManager[aSPI].receiveBuffer.buffer!=NULL )
        {
            if ( data.info.errors.asFlags.collision )
            {
                *(spiManager[aSPI].receiveBuffer.writePointer) = data.info.dataByte;
                spiManager[aSPI].receiveBuffer.writePointer++;
            }
            if ( spiManager[aSPI].receiveBuffer.writePointer == (spiManager[aSPI].receiveBuffer.buffer+spiManager[aSPI].receiveBuffer.length) )
            {
                spiManager[aSPI].state.communicationState=eSPI_idle;
            }
            else
            {
                if ( !Sys_spiTransceiveData(aSPI, 0xff) )
                {
                    spiManager[aSPI].state.communicationState=eSPI_idle;
                }
            }
        }
        break;
    case eSPI_transmitting :
        if ( spiManager[aSPI].transmitBuffer.buffer!=NULL )
        {
            spiManager[aSPI].transmitBuffer.readPointer++;
            if ( spiManager[aSPI].transmitBuffer.readPointer == (spiManager[aSPI].transmitBuffer.buffer+spiManager[aSPI].transmitBuffer.length) )
            {
                spiManager[aSPI].state.communicationState=eSPI_idle;
            }
            else
            {
                if ( !Sys_spiTransceiveData(aSPI, *(spiManager[aSPI].transmitBuffer.readPointer)) )
                {
                    spiManager[aSPI].state.communicationState=eSPI_idle;
                }
            }
        }
        break;
    case eSPI_idle :
        break;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_checkFrequency */
/* EasyCODE F */
static u8 spi_checkFrequency( u16 freq, TSys_SPIPrescaler* outSpeed)
{
    u8 result;
    u32 spiPrescale;
    spiPrescale=_MCU_CLOCK_FREQUENCY_/1000;
    spiPrescale/=freq;
    /* EasyCODE - */
    result=FALSE;
    /* EasyCODE - */
    *outSpeed = eSys_SPIPrescaler2;
    while ( !result && (*outSpeed <=eSys_SPIPrescaler128) )
    {
        if ( spiPrescale <= *outSpeed )
        {
            result = TRUE;
        }
        else
        {
            *outSpeed *= 2;
        }
    }
    return result;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_initializedAndAvailable */
/* EasyCODE F */
static u8 spi_initializedAndAvailable(u8 aSPI)
{
    if ( aSPI < cSys_SPICount )
    {
        return spiManager[aSPI].state.state.asFlags.initialized;
    }
    else
    {
        return FALSE;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_blockingTransmit */
/* EasyCODE F */
static u8 spi_blockingTransmit(u8 aSPI, u8* data, u16 dataLen)
{
    u16 i;
    for ( i=0; i < dataLen; i++ )
    {
        Sys_spiTransceiveData(aSPI, data[i]);
    }
    return TRUE;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_blockingReceive */
/* EasyCODE F */
static u8 spi_blockingReceive(u8 aSPI, u8* data, u16 dataLen)
{
    u16 i;
    for ( i=0; i < dataLen; i++ )
    {
        data[i] = Sys_spiTransceiveData(aSPI, 0xff);
    }
    return TRUE;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_globalInit */
/* EasyCODE F */
static u8 spi_globalInit(u8 aSPI, u16 freq, TSys_SPIMode mode)
{
    TSys_SPIPrescaler speed;
    if ( aSPI < cSys_SPICount )
    {
        if ( !spiManager[aSPI].state.state.asFlags.initialized )
        {
            if ( spi_checkFrequency( freq, &speed) )
            {
                if ( Sys_spiEnable( aSPI) )
                {
                    spiManager[aSPI].state.state.asU8=0;
                    spiManager[aSPI].state.state.asFlags.initialized=1;
                    spiManager[aSPI].state.communicationState = eSPI_idle;
                    Sys_spiSetSpeed(aSPI, speed);
                    Sys_spiSetMode(aSPI, mode);
                    return TRUE;
                }
            }
        }
    }
    return FALSE;
}

/* EasyCODE ) */
/* EasyCODE ] */
/* EasyCODE ( 0 
spi_startUp */
/* EasyCODE F */
void spi_startUp(void)
{
    u8 i;
    for ( i=0; i < cSys_SPICount; i++ )
    {
        spiManager[i].state.state.asU8=0;
        spiManager[i].receiveBuffer.buffer=NULL;
        spiManager[i].receiveBuffer.length=0;
        spiManager[i].transmitBuffer.buffer=NULL;
        spiManager[i].transmitBuffer.length=0;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_maximumFrequency */
/* EasyCODE F */
u16 spi_maximumFrequency(void)
{
    //Main clock frequency is in Hz so divide by 1000
    return _MCU_CLOCK_FREQUENCY_/(eSys_SPIPrescaler2*1000);
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_initBlockingMaster */
/* EasyCODE F */
u8 spi_initBlockingMaster(u8 aSPI, u16 freq, TSys_SPIMode mode)
{
    if ( spi_globalInit(aSPI, freq, mode) )
    {
        spiManager[aSPI].state.operationMode = eSPI_blockingMaster;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_destruct */
/* EasyCODE F */
u8 spi_destruct(u8 aSPI)
{
    if ( spi_initializedAndAvailable(aSPI) )
    {
        return Sys_spiDisable(aSPI);
    }
    else
    {
        return FALSE;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_transmitData */
/* EasyCODE F */
u8 spi_transmitData(u8 aSPI, u8* data, u16 dataLen)
{
    if ( spi_initializedAndAvailable(aSPI) )
    {
        switch ( spiManager[aSPI].state.operationMode )
        {
        case eSPI_blockingMaster :
            return spi_blockingTransmit(aSPI, data, dataLen);
        }
    }
    else
    {
        return FALSE;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_receiveData */
/* EasyCODE F */
u8 spi_receiveData(u8 aSPI, u8* data, u16 dataLen)
{
    if ( spi_initializedAndAvailable(aSPI) )
    {
        switch ( spiManager[aSPI].state.operationMode )
        {
        case eSPI_blockingMaster :
            return spi_blockingReceive(aSPI, data, dataLen);
        }
    }
    else
    {
        return FALSE;
    }
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_changeFrequency */
/* EasyCODE F */
u8 spi_changeFrequency(u8 aSPI, u16 freq)
{
    TSys_SPIPrescaler speed;
    if ( aSPI < cSys_SPICount )
    {
        if ( spiManager[aSPI].state.state.asFlags.initialized )
        {
            if ( spi_checkFrequency( freq, &speed) )
            {
                return Sys_spiSetSpeed(aSPI, speed);
            }
        }
    }
    return FALSE;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_changeMode */
/* EasyCODE F */
u8 spi_changeMode(u8 aSPI, TSys_SPIMode mode)
{
    if ( aSPI < cSys_SPICount )
    {
        if ( spiManager[aSPI].state.state.asFlags.initialized )
        {
            return Sys_spiSetMode(aSPI, mode);
        }
    }
    return FALSE;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_changeDataOrder */
/* EasyCODE F */
u8 spi_changeDataOrder(u8 aSPI, u8 lsbFirst)
{
    if ( aSPI < cSys_SPICount )
    {
        if ( spiManager[aSPI].state.state.asFlags.initialized )
        {
            return Sys_spiSetDataOrder(aSPI, lsbFirst);
        }
    }
    return FALSE;
}

/* EasyCODE ) */
/* EasyCODE ( 0 
spi_changeDataPolarity */
/* EasyCODE F */
u8 spi_changeDataPolarity(u8 aSPI, u8 isInverse)
{
    if ( aSPI < cSys_SPICount )
    {
        if ( spiManager[aSPI].state.state.asFlags.initialized )
        {
            return Sys_spiSetDataPolarity(aSPI, isInverse);
        }
    }
    return FALSE;
}

/* EasyCODE ) */
/* EasyCODE ) */
