/* EasyCODE V8 */
/* EasyCODE ( 0 */
#define cSys_Internal
/* EasyCODE - */
#include <system.h>
#if cSys_SPICount > 0
    #include <sys_cpu.h>
    #include <stdlib.h>
    /* EasyCODE [ 
    TSys_HardwareMap */
    /* EasyCODE C */
    typedef union USys_HardwareMap
    {
            u16 asU16;
            /* EasyCODE C */
            struct SSys_HardwareMap
            {
                    u16 index:	15;
                    u8 valid:	1;
            }asStruct;
            /* EasyCODE E */
    }TSys_HardwareMap;
    /* EasyCODE E */
    /* EasyCODE ] */
    /* EasyCODE [ 
    TSPIHardware */
    /* EasyCODE C */
    typedef struct SSPIHardware
    {
            u8 id;					//ID from sys_config.h
            SPI_t* spiReg;			//ptr to ADC converter register
            u8* mosiPinControl;		//Control register of Data pin
            u8* misoPinControl;		//Control register of Data pin
            PORT_t* portReg;			//ptr to port register
    }TSPIHardware;
    /* EasyCODE E */
    /* EasyCODE ] */
    /* EasyCODE < */
    static flash TSPIHardware spiHardwareArray[cSys_SPICount]=
    {
    /* EasyCODE > */
    /* EasyCODE ( 0 
    SPI Hardware */
    #ifdef HW_SPI_C
        /* EasyCODE :
         */
        {
            HW_SPI_C, &SPIC, &PORTC.PIN5CTRL, &PORTC.PIN6CTRL, &PORTC
        }

        ,
    #endif 
    #ifdef HW_SPI_D
        /* EasyCODE :
         */
        {
            HW_SPI_D, &SPID, &PORTD.PIN5CTRL, &PORTD.PIN6CTRL, &PORTD
        }

        ,
    #endif 
    #ifdef HW_SPI_E
        /* EasyCODE :
         */
        {
            HW_SPI_E, &SPIE, &PORTE.PIN5CTRL, &PORTE.PIN6CTRL, &PORTE
        }

        ,
    #endif 
    /* EasyCODE ) */
    /* EasyCODE < */
    };
    /* EasyCODE > */
    /* EasyCODE [ 
    Private Functions */
    /* EasyCODE ( 0 
    GetSPIHardware */
    /* EasyCODE F */
    static flash TSPIHardware* GetSPIHardware(u8 id)
    {
        #if cSys_SPICount!=1
            u8 index;
        #endif 
        if ( id < cSys_SPICount )
        {
            #if cSys_SPICount==1
                return &spiHardwareArray[0];
            #else 
                index=0;
                while ( (index < cSys_SPICount) && (spiHardwareArray[index].id != id) )
                {
                    index++;
                }
                if ( index < cSys_SPICount )
                {
                    return &spiHardwareArray[index];
                }
                else
                {
                    return NULL;
                }
            #endif 
        }
        else
        {
            return NULL;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ] */
    /* EasyCODE ( 0 
    Sys_spiEnable */
    /* EasyCODE F */
    u8 Sys_spiEnable(u8 aSPI)
    {
        u8 dummy;
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            //Configure SPI:
            //	LSB first
            //	Highest speed (f(CPU)/2)
            //	SPI Mode 0
            spiHardware->spiReg->CTRL = SPI_CLK2X_bm | SPI_MASTER_bm | SPI_DORD_bm;
            /* EasyCODE - */
            //Set master mode: also configures IO
            Sys_spiSetMasterSlave( aSPI, TRUE);
            /* EasyCODE - */
            //disable interrupt
            spiHardware->spiReg->INTCTRL = 0;
            /* EasyCODE - */
            //Enable SPI
            spiHardware->spiReg->CTRL |= SPI_ENABLE_bm;
            if ( (spiHardware->spiReg->STATUS & SPI_IF_bm) || (spiHardware->spiReg->STATUS & SPI_WRCOL_bm) )
            {
                dummy = spiHardware->spiReg->STATUS;
                dummy = spiHardware->spiReg->DATA;
            }
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiDisable */
    /* EasyCODE F */
    u8 Sys_spiDisable(u8 aSPI)
    {
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            //Disable SPI
            spiHardware->spiReg->CTRL = 0;
            /* EasyCODE - */
            //Disable interrupt
            spiHardware->spiReg->INTCTRL = 0;
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiSetSpeed */
    /* EasyCODE F */
    u8 Sys_spiSetSpeed(u8 aSPI, TSys_SPIPrescaler speed)
    {
        u8 result;
        /* EasyCODE - */
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            switch ( speed )
            {
            case eSys_SPIPrescaler2 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL |= SPI_CLK2X_bm;
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV4_gc;
                break;
            case eSys_SPIPrescaler4 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~SPI_CLK2X_bm;
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV4_gc;
                break;
            case eSys_SPIPrescaler8 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL |= SPI_CLK2X_bm;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV16_gc;
                break;
            case eSys_SPIPrescaler16 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~SPI_CLK2X_bm;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV16_gc;
                break;
            case eSys_SPIPrescaler32 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL |= SPI_CLK2X_bm;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV64_gc;
                break;
            case eSys_SPIPrescaler64 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~SPI_CLK2X_bm;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV64_gc;
                break;
            case eSys_SPIPrescaler128 :
                result = TRUE;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~SPI_CLK2X_bm;
                /* EasyCODE - */
                spiHardware->spiReg->CTRL &= ~(SPI_PRESCALER_gm);
                spiHardware->spiReg->CTRL |= SPI_PRESCALER_DIV128_gc;
                break;
            default:
                result =FALSE;
            }
        }
        else
        {
            result =FALSE;
        }
        return result;
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiSetDataOrder */
    /* EasyCODE F */
    u8 Sys_spiSetDataOrder(u8 aSPI, u8 lsbFirst)
    {
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            if ( lsbFirst )
            {
                //LSB first
                spiHardware->spiReg->CTRL |= SPI_DORD_bm;
            }
            else
            {
                //MSB first
                spiHardware->spiReg->CTRL &= ~SPI_DORD_bm;
            }
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiSetMode */
    /* EasyCODE F */
    u8 Sys_spiSetMode(u8 aSPI, TSys_SPIMode mode)
    {
        u8 result=FALSE;
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            //Clear current mode
            spiHardware->spiReg->CTRL &= ~SPI_MODE_gm;
            switch ( mode )
            {
            case eSys_SPIMode0 :
                spiHardware->spiReg->CTRL |= SPI_MODE_0_gc;
                result = TRUE;
                break;
            case eSys_SPIMode1 :
                spiHardware->spiReg->CTRL |= SPI_MODE_1_gc;
                result = TRUE;
                break;
            case eSys_SPIMode2 :
                spiHardware->spiReg->CTRL |= SPI_MODE_2_gc;
                result = TRUE;
                break;
            case eSys_SPIMode3 :
                spiHardware->spiReg->CTRL |= SPI_MODE_3_gc;
                result = TRUE;
                break;
            }
        }
        return result;
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiSetDataPolarity */
    /* EasyCODE F */
    u8 Sys_spiSetDataPolarity(u8 aSPI, u8 isInverse)
    {
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            if ( isInverse )
            {
                *(spiHardware->mosiPinControl) |= PORT_INVEN_bm;
                /* EasyCODE - */
                *(spiHardware->misoPinControl) |= PORT_INVEN_bm;
            }
            else
            {
                *(spiHardware->mosiPinControl) &= ~PORT_INVEN_bm;
                /* EasyCODE - */
                *(spiHardware->misoPinControl) &= ~PORT_INVEN_bm;
            }
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiTransceiveData */
    /* EasyCODE F */
    u8 Sys_spiTransceiveData(u8 aSPI, u8 data)
    {
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        /* EasyCODE - */
        u8 d;
        if ( spiHardware != NULL )
        {
            if ( spiHardware->spiReg->STATUS )
            {
                d=spiHardware->spiReg->DATA;
            }
            spiHardware->spiReg->DATA = data;
            while ( !(spiHardware->spiReg->STATUS & SPI_IF_bm) )
            {
            }
            d=spiHardware->spiReg->DATA;
            return d;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
    /* EasyCODE ( 0 
    Sys_spiSetMasterSlave */
    /* EasyCODE F */
    u8 Sys_spiSetMasterSlave(u8 aSPI, u8 isMaster)
    {
        flash TSPIHardware* spiHardware = GetSPIHardware(aSPI);
        if ( spiHardware != NULL )
        {
            if ( isMaster )
            {
                spiHardware->spiReg->CTRL |= SPI_MASTER_bm;
                /* EasyCODE - */
                //Clock must be output and controlled by application
                spiHardware->portReg->DIRSET = SPI_SS_bm | SPI_SCK_bm;
                /* EasyCODE - */
                //Enable IO
                spiHardware->portReg->DIRSET = SPI_MOSI_bm;
                spiHardware->portReg->DIRCLR = SPI_MISO_bm;
            }
            else
            {
                spiHardware->spiReg->CTRL = ~SPI_MASTER_bm;
                /* EasyCODE - */
                //Chip select and clock must be input and is controlled by HW
                spiHardware->portReg->DIRCLR = SPI_SS_bm | SPI_SCK_bm;
                /* EasyCODE - */
                //Enable IO
                spiHardware->portReg->DIRCLR = SPI_MOSI_bm;
                spiHardware->portReg->DIRSET = SPI_MISO_bm;
            }
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }

    /* EasyCODE ) */
#endif 
/* EasyCODE ) */
